@@ -17,7 +17,6 @@ import os
 import tempfile
 
 import mock
-from oslo_serialization import jsonutils
 import six
 
 from novaclient import api_versions
@@ -38,10 +37,15 @@ class ServersTest(utils.FixturedTestCase):
 
     def setUp(self):
         super(ServersTest, self).setUp()
-        self.useFixture(floatingips.FloatingFixture(self.requests))
+        self.useFixture(floatingips.FloatingFixture(self.requests_mock))
         if self.api_version:
             self.cs.api_version = api_versions.APIVersion(self.api_version)
 
+    def _get_server_create_default_nics(self):
+        """Callback for default nics kwarg when creating a server.
+        """
+        return None
+
     def test_list_servers(self):
         sl = self.cs.servers.list()
         self.assert_request_id(sl, fakes.FAKE_REQUEST_ID_LIST)
@@ -63,8 +67,8 @@ class ServersTest(utils.FixturedTestCase):
 
         self.assertEqual(2, len(sl))
 
-        self.assertEqual(self.requests.request_history[-2].method, 'GET')
-        self.assertEqual(self.requests.request_history[-2].path_url,
+        self.assertEqual(self.requests_mock.request_history[-2].method, 'GET')
+        self.assertEqual(self.requests_mock.request_history[-2].path_url,
                          '/servers/detail?marker=1234')
         self.assert_called('GET', '/servers/detail?marker=5678')
 
@@ -132,7 +136,8 @@ class ServersTest(utils.FixturedTestCase):
             files={
                 '/etc/passwd': 'some data',                 # a file
                 '/tmp/foo.txt': six.StringIO('data'),   # a stream
-            }
+            },
+            nics=self._get_server_create_default_nics()
         )
         self.assert_request_id(s, fakes.FAKE_REQUEST_ID_LIST)
         self.assert_called('POST', '/servers')
@@ -191,7 +196,8 @@ class ServersTest(utils.FixturedTestCase):
                 meta={'foo': 'bar'},
                 userdata="hello moto",
                 key_name="fakekey",
-                block_device_mapping_v2=bdm
+                block_device_mapping_v2=bdm,
+                nics=self._get_server_create_default_nics()
             )
             self.assert_request_id(s, fakes.FAKE_REQUEST_ID_LIST)
             self.assert_called('POST', '/os-volumes_boot')
@@ -239,7 +245,8 @@ class ServersTest(utils.FixturedTestCase):
                 userdata="hello moto",
                 key_name="fakekey",
                 access_ip_v6=access_ip_v6,
-                access_ip_v4=access_ip_v4
+                access_ip_v4=access_ip_v4,
+                nics=self._get_server_create_default_nics()
             )
             self.assert_request_id(s, fakes.FAKE_REQUEST_ID_LIST)
             self.assert_called('POST', '/servers')
@@ -256,6 +263,7 @@ class ServersTest(utils.FixturedTestCase):
                 '/etc/passwd': 'some data',                 # a file
                 '/tmp/foo.txt': six.StringIO('data'),   # a stream
             },
+            nics=self._get_server_create_default_nics(),
         )
         self.assert_request_id(s, fakes.FAKE_REQUEST_ID_LIST)
         self.assert_called('POST', '/servers')
@@ -273,6 +281,7 @@ class ServersTest(utils.FixturedTestCase):
                 '/etc/passwd': 'some data',                 # a file
                 '/tmp/foo.txt': six.StringIO('data'),   # a stream
             },
+            nics=self._get_server_create_default_nics(),
         )
         self.assert_request_id(s, fakes.FAKE_REQUEST_ID_LIST)
         self.assert_called('POST', '/servers')
@@ -290,6 +299,7 @@ class ServersTest(utils.FixturedTestCase):
                 '/etc/passwd': 'some data',                 # a file
                 '/tmp/foo.txt': six.StringIO('data'),   # a stream
             },
+            nics=self._get_server_create_default_nics(),
         )
         self.assert_request_id(s, fakes.FAKE_REQUEST_ID_LIST)
         self.assert_called('POST', '/servers')
@@ -303,12 +313,13 @@ class ServersTest(utils.FixturedTestCase):
             image=1,
             flavor=1,
             admin_pass=test_password,
-            key_name=test_key
+            key_name=test_key,
+            nics=self._get_server_create_default_nics()
         )
         self.assert_request_id(s, fakes.FAKE_REQUEST_ID_LIST)
         self.assert_called('POST', '/servers')
         self.assertIsInstance(s, servers.Server)
-        body = jsonutils.loads(self.requests.last_request.body)
+        body = self.requests_mock.last_request.json()
         self.assertEqual(test_password, body['server']['adminPass'])
 
     def test_create_server_userdata_bin(self):
@@ -328,12 +339,13 @@ class ServersTest(utils.FixturedTestCase):
                     '/etc/passwd': 'some data',                 # a file
                     '/tmp/foo.txt': six.StringIO('data'),   # a stream
                 },
+                nics=self._get_server_create_default_nics(),
             )
             self.assert_request_id(s, fakes.FAKE_REQUEST_ID_LIST)
             self.assert_called('POST', '/servers')
             self.assertIsInstance(s, servers.Server)
             # verify userdata matches original
-            body = jsonutils.loads(self.requests.last_request.body)
+            body = self.requests_mock.last_request.json()
             transferred_data = body['server']['user_data']
             transferred_data = base64.b64decode(transferred_data)
             self.assertEqual(original_data, transferred_data)
@@ -343,15 +355,15 @@ class ServersTest(utils.FixturedTestCase):
             name="My server",
             image=1,
             flavor=1,
-            disk_config=disk_config
+            disk_config=disk_config,
+            nics=self._get_server_create_default_nics(),
         )
         self.assert_request_id(s, fakes.FAKE_REQUEST_ID_LIST)
         self.assert_called('POST', '/servers')
         self.assertIsInstance(s, servers.Server)
 
         # verify disk config param was used in the request:
-        body = jsonutils.loads(self.requests.last_request.body)
-        server = body['server']
+        server = self.requests_mock.last_request.json()['server']
         self.assertIn('OS-DCF:diskConfig', server)
         self.assertEqual(disk_config, server['OS-DCF:diskConfig'])
 
@@ -457,9 +469,7 @@ class ServersTest(utils.FixturedTestCase):
         self.assert_called('POST', '/servers/1234/action')
 
         # verify disk config param was used in the request:
-        body = jsonutils.loads(self.requests.last_request.body)
-
-        d = body[operation]
+        d = self.requests_mock.last_request.json()[operation]
         self.assertIn('OS-DCF:diskConfig', d)
         self.assertEqual(disk_config, d['OS-DCF:diskConfig'])
 
@@ -474,8 +484,7 @@ class ServersTest(utils.FixturedTestCase):
         ret = s.rebuild(image=1, preserve_ephemeral=True)
         self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
         self.assert_called('POST', '/servers/1234/action')
-        body = jsonutils.loads(self.requests.last_request.body)
-        d = body['rebuild']
+        d = self.requests_mock.last_request.json()['rebuild']
         self.assertIn('preserve_ephemeral', d)
         self.assertTrue(d['preserve_ephemeral'])
 
@@ -484,8 +493,7 @@ class ServersTest(utils.FixturedTestCase):
         s = self.cs.servers.get(1234)
         ret = s.rebuild(image=1, name='new', meta={'foo': 'bar'}, files=files)
         self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
-        body = jsonutils.loads(self.requests.last_request.body)
-        d = body['rebuild']
+        d = self.requests_mock.last_request.json()['rebuild']
         self.assertEqual('new', d['name'])
         self.assertEqual({'foo': 'bar'}, d['metadata'])
         self.assertEqual('/etc/passwd',
@@ -976,6 +984,16 @@ class ServersTest(utils.FixturedTestCase):
                           key_name="fakekey"
                           )
 
+    def test_create_server_with_nics_auto(self):
+        """Negative test for specifying nics='auto' before 2.37
+        """
+        self.assertRaises(ValueError,
+                          self.cs.servers.create,
+                          name='test',
+                          image='d9d8d53c-4b4a-4144-a5e5-b30d9f1fe46a',
+                          flavor='1',
+                          nics='auto')
+
 
 class ServersV26Test(ServersTest):
 
@@ -1074,7 +1092,8 @@ class ServersV219Test(ServersV217Test):
             flavor=1,
             meta={'foo': 'bar'},
             userdata="hello moto",
-            key_name="fakekey"
+            key_name="fakekey",
+            nics=self._get_server_create_default_nics()
         )
         self.assert_called('POST', '/servers')
 
@@ -1260,3 +1279,39 @@ class ServersV232Test(ServersV226Test):
                           image=1, flavor=1, meta={'foo': 'bar'},
                           userdata="hello moto", key_name="fakekey",
                           block_device_mapping_v2=bdm)
+
+
+class ServersV2_37Test(ServersV226Test):
+
+    api_version = "2.37"
+
+    def _get_server_create_default_nics(self):
+        return 'auto'
+
+    def test_create_server_no_nics(self):
+        """Tests that nics are required in microversion 2.37+
+        """
+        self.assertRaises(ValueError, self.cs.servers.create,
+                          name='test',
+                          image='d9d8d53c-4b4a-4144-a5e5-b30d9f1fe46a',
+                          flavor='1')
+
+    def test_create_server_with_nics_auto(self):
+        s = self.cs.servers.create(
+            name='test', image='d9d8d53c-4b4a-4144-a5e5-b30d9f1fe46a',
+            flavor='1', nics=self._get_server_create_default_nics())
+        self.assert_request_id(s, fakes.FAKE_REQUEST_ID_LIST)
+        self.assert_called('POST', '/servers')
+        self.assertIsInstance(s, servers.Server)
+
+    def test_add_floating_ip(self):
+        # self.cs.floating_ips.list() is not available after 2.35
+        pass
+
+    def test_add_floating_ip_to_fixed(self):
+        # self.cs.floating_ips.list() is not available after 2.35
+        pass
+
+    def test_remove_floating_ip(self):
+        # self.cs.floating_ips.list() is not available after 2.35
+        pass
